from ...abstasks.AbsTaskSTS import AbsTaskSTS


class BiossesSTS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "BIOSSES",
            "hf_hub_name": "mteb/biosses-sts",
            "description": "Biomedical Semantic Similarity Estimation.",
            "reference": "https://tabilab.cmpe.boun.edu.tr/BIOSSES/DataSet.html",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 4,
            "revision": "d3fb88f8f02e40887cd149695127462bbcf29b4a",
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class EightTagsClustering(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "8TagsClustering",
            "hf_hub_name": "PL-MTEB/8tags-clustering",
            "description": "Clustering of headlines from social media posts in Polish belonging to 8 categories: film, history, "
                           "food, medicine, motorization, work, sport and technology.",
            "reference": "https://aclanthology.org/2020.lrec-1.207.pdf",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "v_measure"
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class RedditClusteringP2P(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "RedditClusteringP2P",
            "hf_hub_name": "mteb/reddit-clustering-p2p",
            "description": (
                "Clustering of title+posts from reddit. Clustering of 10 sets with 1K - 100K samples and 10 - 100 labels each."
            ),
            "reference": "https://huggingface.co/datasets/sentence-transformers/reddit-title-body",
            "type": "Clustering",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "282350215ef01743dc01b456c7f5241fa8937f16",
        }



import datasets

from mteb.abstasks import AbsTaskClassification


class ScalaDaClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "ScalaDaClassification",
            "hf_hub_name": "ScandEval/scala-da",
            "description": "A modified version of DDT modified for linguistic acceptability classification",
            "reference": "https://aclanthology.org/2023.nodalida-1.20/",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["da"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "1de08520a7b361e92ffa2a2201ebd41942c54675",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"], revision=self.description.get("revision", None)
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        # convert label to a 0/1 label
        labels = self.dataset["train"]["label"]  # type: ignore
        lab2idx = {lab: idx for idx, lab in enumerate(set(labels))}
        self.dataset = self.dataset.map(lambda x: {"label": lab2idx[x["label"]]}, remove_columns=["label"])


class ScalaNbClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "ScalaNbClassification",
            "hf_hub_name": "ScandEval/scala-nb",
            "description": "A Norwegian dataset for linguistic acceptability classification for Bokmål",
            "reference": "https://aclanthology.org/2023.nodalida-1.20/",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["no", "nb"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "237111a078ad5a834a55c57803d40bbe410ed03b",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"], revision=self.description.get("revision", None)
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        # convert label to a 0/1 label
        labels = self.dataset["train"]["label"]  # type: ignore
        lab2idx = {lab: idx for idx, lab in enumerate(set(labels))}
        self.dataset = self.dataset.map(lambda x: {"label": lab2idx[x["label"]]}, remove_columns=["label"])


class ScalaNnClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "ScalaNbClassification",
            "hf_hub_name": "ScandEval/scala-nn",
            "description": "A Norwegian dataset for linguistic acceptability classification for Nynorsk",
            "reference": "https://aclanthology.org/2023.nodalida-1.20/",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["no", "nn"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "9d9a2a4092ed3cacf0744592f6d2f32ab8ef4c0b",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"], revision=self.description.get("revision", None)
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        # convert label to a 0/1 label
        labels = self.dataset["train"]["label"]  # type: ignore
        lab2idx = {lab: idx for idx, lab in enumerate(set(labels))}
        self.dataset = self.dataset.map(lambda x: {"label": lab2idx[x["label"]]}, remove_columns=["label"])


class ScalaSvClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "ScalaSvClassification",
            "hf_hub_name": "ScandEval/scala-sv",
            "description": "A Swedish dataset for linguistic acceptability classification",
            "reference": "https://aclanthology.org/2023.nodalida-1.20/",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["sv"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "1b48e3dcb02872335ff985ff938a054a4ed99008",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"], revision=self.description.get("revision", None)
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        # convert label to a 0/1 label
        labels = self.dataset["train"]["label"]  # type: ignore
        lab2idx = {lab: idx for idx, lab in enumerate(set(labels))}
        self.dataset = self.dataset.map(lambda x: {"label": lab2idx[x["label"]]}, remove_columns=["label"])



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class RedditClustering(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "RedditClustering",
            "hf_hub_name": "mteb/reddit-clustering",
            "description": (
                "Clustering of titles from 199 subreddits. Clustering of 25 sets, each with 10-50 classes, and each"
                " class with 100 - 1000 sentences."
            ),
            "reference": "https://arxiv.org/abs/2104.07081",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "24640382cdbf8abc73003fb0fa6d111a705499eb",
        }



from ...abstasks.AbsTaskReranking import AbsTaskReranking


class MindSmallReranking(AbsTaskReranking):
    @property
    def description(self):
        return {
            "name": "MindSmallReranking",
            "hf_hub_name": "mteb/mind_small",
            "description": "Microsoft News Dataset: A Large-Scale English Dataset for News Recommendation Research",
            "reference": "https://msnews.github.io/assets/doc/ACL2020_MIND.pdf",
            "type": "Reranking",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "map",
            "revision": "3bdac13927fdc888b903db93b2ffdbd90b295a69",
        }



from ...abstasks import AbsTaskSTS, CrosslingualTask

_LANGUAGES = ["ko-ko", "ar-ar", "en-ar", "en-de", "en-en", "en-tr", "es-en", "es-es", "fr-en", "it-en", "nl-en"]


class STS17Crosslingual(AbsTaskSTS, CrosslingualTask):
    @property
    def description(self):
        return {
            "name": "STS17",
            "hf_hub_name": "mteb/sts17-crosslingual-sts",
            "description": "STS 2017 dataset",
            "reference": "http://alt.qcri.org/semeval2016/task1/",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": _LANGUAGES,
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
            "revision": "af5e6fb845001ecf41f4c1e033ce921939a2a68d",
        }



from collections import defaultdict
from datasets import load_dataset, DatasetDict

from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval


def load_retrieval_data(hf_hub_name, eval_splits):
    eval_split = eval_splits[0]
    dataset = load_dataset(hf_hub_name)
    qrels = load_dataset(hf_hub_name + '-qrels')[eval_split]

    corpus = {e['id']: {'text': e['text']} for e in dataset['corpus']}
    queries = {e['id']: e['text'] for e in dataset['queries']}
    relevant_docs = defaultdict(dict)
    for e in qrels:
        relevant_docs[e['qid']][e['pid']] = e['score']

    corpus = DatasetDict({eval_split:corpus})
    queries = DatasetDict({eval_split:queries})
    relevant_docs = DatasetDict({eval_split:relevant_docs})
    return corpus, queries, relevant_docs


class T2Retrieval(AbsTaskRetrieval):
    @property
    def description(self):
        return {
            'name': 'T2Retrieval',
            'hf_hub_name': 'C-MTEB/T2Retrieval',
            'reference': 'https://arxiv.org/abs/2304.03679',
            'description': 'T2Ranking: A large-scale Chinese Benchmark for Passage Ranking',
            'type': 'Retrieval',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'ndcg_at_10',
        }

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(self.description['hf_hub_name'],
                                                                            self.description['eval_splits'])
        self.data_loaded = True


class MMarcoRetrieval(AbsTaskRetrieval):
    @property
    def description(self):
        return {
            'name': 'MMarcoRetrieval',
            'hf_hub_name': 'C-MTEB/MMarcoRetrieval',
            'reference': 'https://github.com/unicamp-dl/mMARCO',
            'description': 'mMARCO is a multilingual version of the MS MARCO passage ranking dataset',
            'type': 'Retrieval',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'ndcg_at_10',
        }

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(self.description['hf_hub_name'],
                                                                            self.description['eval_splits'])
        self.data_loaded = True


class DuRetrieval(AbsTaskRetrieval):
    @property
    def description(self):
        return {
            'name': 'DuRetrieval',
            'hf_hub_name': 'C-MTEB/DuRetrieval',
            'reference': 'https://aclanthology.org/2022.emnlp-main.357.pdf',
            'description': 'A Large-scale Chinese Benchmark for Passage Retrieval from Web Search Engine',
            'type': 'Retrieval',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'ndcg_at_10',
        }

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(self.description['hf_hub_name'],
                                                                            self.description['eval_splits'])
        self.data_loaded = True


class CovidRetrieval(AbsTaskRetrieval):
    @property
    def description(self):
        return {
            'name': 'CovidRetrieval',
            'hf_hub_name': 'C-MTEB/CovidRetrieval',
            'reference': 'https://aclanthology.org/2022.emnlp-main.357.pdf',
            'description': 'COVID-19 news articles',
            'type': 'Retrieval',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'ndcg_at_10',
        }

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(self.description['hf_hub_name'],
                                                                            self.description['eval_splits'])
        self.data_loaded = True



class CmedqaRetrieval(AbsTaskRetrieval):
    @property
    def description(self):
        return {
            'name': 'CmedqaRetrieval',
            'hf_hub_name': 'C-MTEB/CmedqaRetrieval',
            'reference': 'https://aclanthology.org/2022.emnlp-main.357.pdf',
            'description': 'Online medical consultation text',
            'type': 'Retrieval',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'ndcg_at_10',
        }

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(self.description['hf_hub_name'],
                                                                            self.description['eval_splits'])
        self.data_loaded = True


class EcomRetrieval(AbsTaskRetrieval):
    @property
    def description(self):
        return {
            'name': 'EcomRetrieval',
            'hf_hub_name': 'C-MTEB/EcomRetrieval',
            'reference': 'https://arxiv.org/abs/2203.03367',
            'description': 'Passage retrieval dataset collected from Alibaba search engine systems in ecom domain',
            'type': 'Retrieval',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'ndcg_at_10',
        }

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(self.description['hf_hub_name'],
                                                                            self.description['eval_splits'])
        self.data_loaded = True


class MedicalRetrieval(AbsTaskRetrieval):
    @property
    def description(self):
        return {
            'name': 'MedicalRetrieval',
            'hf_hub_name': 'C-MTEB/MedicalRetrieval',
            'reference': 'https://arxiv.org/abs/2203.03367',
            'description': 'Passage retrieval dataset collected from Alibaba search engine systems in medical domain',
            'type': 'Retrieval',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'ndcg_at_10',
        }

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(self.description['hf_hub_name'],
                                                                            self.description['eval_splits'])
        self.data_loaded = True


class VideoRetrieval(AbsTaskRetrieval):
    @property
    def description(self):
        return {
            'name': 'VideoRetrieval',
            'hf_hub_name': 'C-MTEB/VideoRetrieval',
            'reference': 'https://arxiv.org/abs/2203.03367',
            'description': 'Passage retrieval dataset collected from Alibaba search engine systems in video domain',
            'type': 'Retrieval',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'ndcg_at_10',
        }

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(self.description['hf_hub_name'],
                                                                            self.description['eval_splits'])
        self.data_loaded = True




from ...abstasks.AbsTaskClustering import AbsTaskClustering


class CLSClusteringS2S(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "CLSClusteringS2S",
            "hf_hub_name": "C-MTEB/CLSClusteringS2S",
            "description": (
                "Clustering of titles from CLS dataset. Clustering of 13 sets, based on the main category."
            ),
            "reference": "https://arxiv.org/abs/2209.05034",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "v_measure",
        }



class CLSClusteringP2P(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "CLSClusteringP2P",
            "hf_hub_name": "C-MTEB/CLSClusteringP2P",
            "description": (
                "Clustering of titles + abstract from CLS dataset. Clustering of 13 sets, based on the main category."
            ),
            "reference": "https://arxiv.org/abs/2209.05034",
            "type": "Clustering",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "v_measure",
        }



class ThuNewsClusteringS2S(AbsTaskClustering):
    @property
    def description(self):
        return {
            'name': 'ThuNewsClusteringS2S',
            'hf_hub_name': 'C-MTEB/ThuNewsClusteringS2S',
            'description': 'Clustering of titles from the THUCNews dataset',
            "reference": "http://thuctc.thunlp.org/",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "v_measure",
        }


class ThuNewsClusteringP2P(AbsTaskClustering):
    @property
    def description(self):
        return {
            'name': 'ThuNewsClusteringP2P',
            'hf_hub_name': 'C-MTEB/ThuNewsClusteringP2P',
            'description': 'Clustering of titles + abstracts from the THUCNews dataset',
            "reference": "http://thuctc.thunlp.org/",
            "type": "Clustering",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "v_measure",
        }



from .BiossesSTS import *
from .SickrSTS import *
from .STS12STS import *
from .STS13STS import *
from .STS14STS import *
from .STS15STS import *
from .STS16STS import *
from .STS17CrosslingualSTS import *
from .STS22CrosslingualSTS import *
from .STSBenchmarkSTS import *
from .CMTEBSTS import *
from .PolishSTS import *



import datasets

from mteb.abstasks import AbsTaskClassification


class DanishPoliticalCommentsClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "DanishPoliticalCommentsClassification",
            "hf_hub_name": "danish_political_comments",
            "description": "A dataset of Danish political comments rated for sentiment",
            "reference": "NA",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["train"],
            "eval_langs": ["da"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "edbb03726c04a0efab14fc8c3b8b79e4d420e5a1",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"], revision=self.description.get("revision")
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        self.dataset = self.dataset.rename_column("sentence", "text")
        self.dataset = self.dataset.rename_column("target", "label")

        # create train and test splits
        self.dataset = self.dataset["train"].train_test_split(0.2, seed=self.seed)



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRPLTask import BeIRPLTask


class MSMARCOPL(AbsTaskRetrieval, BeIRPLTask):
    @property
    def description(self):
        return {
            "name": "MSMARCO-PL",
            "beir_name": "msmarco-pl",
            "description": "MS MARCO is a collection of datasets focused on deep learning in search",
            "reference": "https://microsoft.github.io/msmarco/",
            "benchmark": "BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["validation", "test"],  # "validation" if using latest BEIR i.e. HFDataLoader
            "eval_langs": ["pl"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class FiQA2018(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "FiQA2018",
            "beir_name": "fiqa",
            "description": "Financial Opinion Mining and Question Answering",
            "reference": "https://sites.google.com/view/fiqa/",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks import AbsTaskClassification, MultilingualTask

_LANGUAGES = [
    "af",
    "am",
    "ar",
    "az",
    "bn",
    "cy",
    "da",
    "de",
    "el",
    "en",
    "es",
    "fa",
    "fi",
    "fr",
    "he",
    "hi",
    "hu",
    "hy",
    "id",
    "is",
    "it",
    "ja",
    "jv",
    "ka",
    "km",
    "kn",
    "ko",
    "lv",
    "ml",
    "mn",
    "ms",
    "my",
    "nb",
    "nl",
    "pl",
    "pt",
    "ro",
    "ru",
    "sl",
    "sq",
    "sv",
    "sw",
    "ta",
    "te",
    "th",
    "tl",
    "tr",
    "ur",
    "vi",
    "zh-CN",
    "zh-TW",
]


class MassiveScenarioClassification(MultilingualTask, AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "MassiveScenarioClassification",
            "hf_hub_name": "mteb/amazon_massive_scenario",
            "description": (
                "MASSIVE: A 1M-Example Multilingual Natural Language Understanding Dataset with 51"
                " Typologically-Diverse Languages"
            ),
            "reference": "https://arxiv.org/abs/2204.08582#:~:text=MASSIVE%20contains%201M%20realistic%2C%20parallel,diverse%20languages%20from%2029%20genera.",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["validation", "test"],
            "eval_langs": _LANGUAGES,
            "main_score": "accuracy",
            "revision": "7d571f92784cd94a019292a1f45445077d0ef634",
        }



from .ArxivClusteringP2P import *
from .ArxivClusteringS2S import *
from .BiorxivClusteringP2P import *
from .BiorxivClusteringS2S import *
from .BlurbsClusteringP2P import *
from .BlurbsClusteringS2S import *
from .MedrxivClusteringP2P import *
from .MedrxivClusteringS2S import *
from .RedditClustering import *
from .RedditClusteringP2P import *
from .StackExchangeClustering import *
from .StackExchangeClusteringP2P import *
from .TenKGnadClusteringP2P import *
from .TenKGnadClusteringS2S import *
from .TwentyNewsgroupsClustering import *
from .CMTEBClustering import *
from .PolishClustering import *



from ...abstasks import AbsTaskClassification


class AmazonPolarityClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "AmazonPolarityClassification",
            "hf_hub_name": "mteb/amazon_polarity",
            "description": "Amazon Polarity Classification Dataset.",
            "reference": "https://dl.acm.org/doi/10.1145/2507157.2507163",
            "category": "p2p",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "accuracy",
            "revision": "e2d317d38cd51312af73b3d32a06d1a08b442046",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class SciFact(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "SciFact",
            "beir_name": "scifact",
            "description": "SciFact verifies scientific claims using evidence from the research literature containing scientific paper abstracts.",
            "reference": "https://github.com/allenai/scifact",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class STS16STS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "STS16",
            "hf_hub_name": "mteb/sts16-sts",
            "description": "SemEval STS 2016 dataset",
            "reference": "http://alt.qcri.org/semeval2016/task1/",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
            "revision": "4d8694f8f0e0100860b497b999b3dbed754a0513",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRPLTask import BeIRPLTask


class QuoraPLRetrieval(AbsTaskRetrieval, BeIRPLTask):
    @property
    def description(self):
        return {
            "name": "Quora-PL",
            "beir_name": "quora-pl",
            "description": (
                "QuoraRetrieval is based on questions that are marked as duplicates on the Quora platform. Given a"
                " question, find other (duplicate) questions."
            ),
            "reference": "https://quoradata.quora.com/First-Quora-Dataset-Release-Question-Pairs",
            "type": "Retrieval",
            "benchmark": "BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language",
            "category": "s2s",
            "eval_splits": ["validation", "test"],  # validation for new DataLoader
            "eval_langs": ["pl"],
            "main_score": "ndcg_at_10",
        }



from .AmazonCounterfactualClassification import *
from .AmazonPolarityClassification import *
from .AmazonReviewsClassification import *
from .AngryTweetsClassification import *
from .Banking77Classification import *
from .DalajClassification import *
from .DanishPoliticalCommentsClassification import *
from .DKHateClassification import *
from .EmotionClassification import *
from .ImdbClassification import *
from .LccSentimentClassification import *
from .MassiveIntentClassification import *
from .MassiveScenarioClassification import *
from .MTOPDomainClassification import *
from .MTOPIntentClassification import *
from .NoRecClassification import *
from .NordicLangClassification import *
from .NorwegianParliamentClassification import *
from .ScalaClassification import *
from .SweRecClassification import *
from .ToxicConversationsClassification import *
from .TweetSentimentExtractionClassification import *
from .CMTEBClassification import *
from .PolishClassification import *



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class StackExchangeClusteringP2P(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "StackExchangeClusteringP2P",
            "hf_hub_name": "mteb/stackexchange-clustering-p2p",
            "description": (
                "Clustering of title+body from stackexchange. Clustering of 5 sets of 10k paragraphs and 5 sets of 5k"
                " paragraphs."
            ),
            "reference": "https://huggingface.co/datasets/flax-sentence-embeddings/stackexchange_title_body_jsonl",
            "type": "Clustering",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "815ca46b2622cec33ccafc3735d572c266efdb44",
        }



from mteb.abstasks import AbsTaskClassification


class NoRecClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "NoRecClassification",
            "hf_hub_name": "ScandEval/norec-mini",  # Using the mini version to keep results ~comparable to the ScandEval benchmark
            "description": "A Norwegian dataset for sentiment classification on review",
            "reference": "https://aclanthology.org/L18-1661/",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["nb"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "07b99ab3363c2e7f8f87015b01c21f4d9b917ce3",
        }



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class STS12STS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "STS12",
            "hf_hub_name": "mteb/sts12-sts",
            "description": "SemEval STS 2012 dataset.",
            "reference": "https://www.aclweb.org/anthology/S12-1051.pdf",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
            "revision": "a0d554a64d88156834ff5ae9920b964011b16384",
        }



from .ArguAnaRetrieval import *
from .ClimateFEVERRetrieval import *
from .CQADupstackAndroidRetrieval import *
from .CQADupstackEnglishRetrieval import *
from .CQADupstackGamingRetrieval import *
from .CQADupstackGisRetrieval import *
from .CQADupstackMathematicaRetrieval import *
from .CQADupstackPhysicsRetrieval import *
from .CQADupstackProgrammersRetrieval import *
from .CQADupstackStatsRetrieval import *
from .CQADupstackTexRetrieval import *
from .CQADupstackUnixRetrieval import *
from .CQADupstackWebmastersRetrieval import *
from .CQADupstackWordpressRetrieval import *
from .DBPediaRetrieval import *
from .FEVERRetrieval import *
from .FiQA2018Retrieval import *
from .HotpotQARetrieval import *
from .MSMARCORetrieval import *
from .MSMARCOv2Retrieval import *
from .NFCorpusRetrieval import *
from .NQRetrieval import *
from .QuoraRetrieval import *
from .SCIDOCSRetrieval import *
from .SciFactRetrieval import *
from .Touche2020Retrieval import *
from .TRECCOVIDRetrieval import *
from .CMTEBRetrieval import *
from .ArguAnaPLRetrieval import *
from .DBPediaPLRetrieval import *
from .FiQAPLRetrieval import *
from .HotpotQAPLRetrieval import *
from .MSMARCOPLRetrieval import *
from .NFCorpusPLRetrieval import *
from .NQPLRetrieval import *
from .QuoraPLRetrieval import *
from .SCIDOCSPLRetrieval import *
from .SciFactPLRetrieval import *
from .TRECCOVIDPLRetrieval import *



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class CQADupstackAndroidRetrieval(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "CQADupstackAndroidRetrieval",
            "beir_name": "cqadupstack/android",
            "description": "CQADupStack: A Benchmark Data Set for Community Question-Answering Research",
            "reference": "http://nlp.cis.unimelb.edu.au/resources/cqadupstack/",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks import AbsTaskClassification, MultilingualTask

_LANGUAGES = ["en", "de", "en-ext", "ja"]


class AmazonCounterfactualClassification(MultilingualTask, AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "AmazonCounterfactualClassification",
            "hf_hub_name": "mteb/amazon_counterfactual",
            "description": (
                "A collection of Amazon customer reviews annotated for counterfactual detection pair classification."
            ),
            "reference": "https://arxiv.org/abs/2104.06893",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["validation", "test"],
            "eval_langs": _LANGUAGES,
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 32,
            "revision": "e8379541af4e31359cca9fbcf4b00f2671dba205",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRPLTask import BeIRPLTask


class FiQAPLRetrieval(AbsTaskRetrieval, BeIRPLTask):
    @property
    def description(self):
        return {
            "name": "FiQA-PL",
            "beir_name": "fiqa-pl",
            "description": "Financial Opinion Mining and Question Answering",
            "reference": "https://sites.google.com/view/fiqa/",
            "benchmark": "BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks import AbsTaskClassification


class EmotionClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "EmotionClassification",
            "hf_hub_name": "mteb/emotion",
            "description": (
                "Emotion is a dataset of English Twitter messages with six basic emotions: anger, fear, joy, love,"
                " sadness, and surprise. For more detailed information please refer to the paper."
            ),
            "reference": "https://www.aclweb.org/anthology/D18-1404",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["validation", "test"],
            "eval_langs": ["en"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "4f58c6b202a23cf9a4da393831edf4f9183cad37",
        }



from mteb.abstasks import AbsTaskClassification


class NorwegianParliamentClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "NorwegianParliament",
            "hf_hub_name": "NbAiLab/norwegian_parliament",
            "description": "Norwegian parliament speeches annotated for sentiment",
            "reference": "https://huggingface.co/datasets/NbAiLab/norwegian_parliament",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test", "validation"],
            "eval_langs": ["nb"],  # assumed to be bokmål
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "f7393532774c66312378d30b197610b43d751972",
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class StackExchangeClustering(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "StackExchangeClustering",
            "hf_hub_name": "mteb/stackexchange-clustering",
            "description": (
                "Clustering of titles from 121 stackexchanges. Clustering of 25 sets, each with 10-50 classes, and"
                " each class with 100 - 1000 sentences."
            ),
            "reference": "https://arxiv.org/abs/2104.07081",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["validation", "test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "6cbc1f7b2bc0622f2e39d2c77fa502909748c259",
        }



from ...abstasks.AbsTaskReranking import AbsTaskReranking


class T2Reranking(AbsTaskReranking):
    @property
    def description(self):
        return {
            'name': 'T2Reranking',
            'hf_hub_name': "C-MTEB/T2Reranking",
            'description': 'T2Ranking: A large-scale Chinese Benchmark for Passage Ranking',
            "reference": "https://arxiv.org/abs/2304.03679",
            'type': 'Reranking',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'map',
        }


class MMarcoReranking(AbsTaskReranking):
    @property
    def description(self):
        return {
            'name': 'MMarcoReranking',
            'hf_hub_name': "C-MTEB/Mmarco-reranking",
            'description': 'mMARCO is a multilingual version of the MS MARCO passage ranking dataset',
            "reference": "https://github.com/unicamp-dl/mMARCO",
            'type': 'Reranking',
            'category': 's2p',
            'eval_splits': ['dev'],
            'eval_langs': ['zh'],
            'main_score': 'map',
        }


class CMedQAv1(AbsTaskReranking):
    @property
    def description(self):
        return {
            'name': 'CMedQAv1',
            "hf_hub_name": "C-MTEB/CMedQAv1-reranking",
            'description': 'Chinese community medical question answering',
            "reference": "https://github.com/zhangsheng93/cMedQA",
            'type': 'Reranking',
            'category': 's2p',
            'eval_splits': ['test'],
            'eval_langs': ['zh'],
            'main_score': 'map',
        }


class CMedQAv2(AbsTaskReranking):
    @property
    def description(self):
        return {
            'name': 'CMedQAv2',
            "hf_hub_name": "C-MTEB/CMedQAv2-reranking",
            'description': 'Chinese community medical question answering',
            "reference": "https://github.com/zhangsheng93/cMedQA2",
            'type': 'Reranking',
            'category': 's2p',
            'eval_splits': ['test'],
            'eval_langs': ['zh'],
            'main_score': 'map',
        }



from ...abstasks import AbsTaskSummarization


class SummEvalSummarization(AbsTaskSummarization):
    @property
    def description(self):
        return {
            "name": "SummEval",
            "hf_hub_name": "mteb/summeval",
            "description": "News Article Summary Semantic Similarity Estimation.",
            "reference": "https://tabilab.cmpe.boun.edu.tr/BIOSSES/DataSet.html",
            "type": "Summarization",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
            "revision": "cda12ad7615edc362dbf25a00fdd61d3b1eaf93c",
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class TenKGnadClusteringS2S(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "TenKGnadClusteringS2S",
            "hf_hub_name": "slvnwhrl/tenkgnad-clustering-s2s",
            "description": "Clustering of news article titles. Clustering of 10 splits on the news article category.",
            "reference": "https://tblock.github.io/10kGNAD/",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["de"],
            "main_score": "v_measure",
            "revision": "6cddbe003f12b9b140aec477b583ac4191f01786",
        }



from ...abstasks.AbsTaskPairClassification import AbsTaskPairClassification


class SprintDuplicateQuestionsPC(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            "name": "SprintDuplicateQuestions",
            "hf_hub_name": "mteb/sprintduplicatequestions-pairclassification",
            "description": "Duplicate questions from the Sprint community.",
            "reference": "https://www.aclweb.org/anthology/D18-1131/",
            "category": "s2s",
            "type": "PairClassification",
            "eval_splits": ["validation", "test"],
            "eval_langs": ["en"],
            "main_score": "ap",
            "revision": "d66bd1f72af766a5cc4b0ca5e00c162f89e8cc46",
        }



import logging
import inspect
from typing import Callable, Dict, List, Set

import numpy as np
import torch
from tqdm import trange

from .Evaluator import Evaluator
from .utils import cos_sim, dot_score

logger = logging.getLogger(__name__)


class RetrievalEvaluator(Evaluator):
    """
    This class evaluates an Information Retrieval (IR) setting.
    Given a set of queries and a large corpus set. It will retrieve for each query the top-k most similar document. It measures
    Mean Reciprocal Rank (MRR), Recall@k, and Normalized Discounted Cumulative Gain (NDCG)
    """

    def __init__(
        self,
        queries: Dict[str, str],  # qid => query
        corpus: Dict[str, str],  # cid => doc
        relevant_docs: Dict[str, Set[str]],  # qid => Set[cid]
        corpus_chunk_size: int = 50000,
        mrr_at_k: List[int] = [10],
        ndcg_at_k: List[int] = [10],
        accuracy_at_k: List[int] = [1, 3, 5, 10],
        precision_recall_at_k: List[int] = [1, 3, 5, 10],
        map_at_k: List[int] = [100],
        show_progress_bar: bool = False,
        batch_size: int = 32,
        name: str = "",
        score_functions: List[Callable[[torch.Tensor, torch.Tensor], torch.Tensor]] = {
            "cos_sim": cos_sim,
            "dot": dot_score,
        },  # Score function, higher=more similar
        main_score_function: str = None,
        limit: int = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.queries_ids = []
        for qid in queries:
            if qid in relevant_docs and len(relevant_docs[qid]) > 0:
                self.queries_ids.append(qid)
                if limit and len(self.queries_ids) >= limit:
                    break

        self.queries = [queries[qid] for qid in self.queries_ids]

        self.corpus_ids = list(corpus.keys())
        self.corpus = [corpus[cid] for cid in self.corpus_ids]

        self.relevant_docs = relevant_docs
        self.corpus_chunk_size = corpus_chunk_size
        self.mrr_at_k = mrr_at_k
        self.ndcg_at_k = ndcg_at_k
        self.accuracy_at_k = accuracy_at_k
        self.precision_recall_at_k = precision_recall_at_k
        self.map_at_k = map_at_k

        self.show_progress_bar = show_progress_bar
        self.batch_size = batch_size
        self.name = name
        self.score_functions = score_functions
        self.score_function_names = sorted(list(self.score_functions.keys()))
        self.main_score_function = main_score_function

    def __call__(self, model) -> float:
        scores = self.compute_metrics(model)

        if self.main_score_function is None:
            scores["main_score"] = max(
                [scores[name]["map_at_" + str(max(self.map_at_k))] for name in self.score_function_names]
            )
        else:
            scores["main_score"] = scores[self.main_score_function]["map_at_" + str(max(self.map_at_k))]
        return scores

    def compute_metrics(self, model, corpus_model=None, corpus_embeddings: torch.Tensor = None) -> Dict[str, float]:
        if corpus_model is None:
            corpus_model = model

        max_k = max(
            max(self.mrr_at_k),
            max(self.ndcg_at_k),
            max(self.accuracy_at_k),
            max(self.precision_recall_at_k),
            max(self.map_at_k),
        )

        # Compute embedding for the queries
        logger.info("Encoding the queries...")
        # We don't know if encode has the kwargs show_progress_bar
        kwargs = {
            "show_progress_bar": self.show_progress_bar
        } if "show_progress_bar" in inspect.signature(model.encode).parameters else {}
        query_embeddings = np.asarray(model.encode(self.queries, batch_size=self.batch_size, **kwargs))
        queries_result_list = {}
        for name in self.score_functions:
            queries_result_list[name] = [[] for _ in range(len(query_embeddings))]

        # Iterate over chunks of the corpus
        logger.info("Encoding chunks of corpus, and computing similarity scores with queries...")
        for corpus_start_idx in trange(
            0,
            len(self.corpus),
            self.corpus_chunk_size,
            desc="Corpus Chunks",
            disable=not self.show_progress_bar,
        ):
            # Encode chunk of corpus
            if corpus_embeddings is None:
                corpus_end_idx = min(corpus_start_idx + self.corpus_chunk_size, len(self.corpus))
                sub_corpus_embeddings = np.asarray(corpus_model.encode(
                    self.corpus[corpus_start_idx:corpus_end_idx],
                    batch_size=self.batch_size,
                ))
            else:
                corpus_end_idx = min(corpus_start_idx + self.corpus_chunk_size, len(corpus_embeddings))
                sub_corpus_embeddings = corpus_embeddings[corpus_start_idx:corpus_end_idx]

            # Compute cosine similarites
            for name, score_function in self.score_functions.items():
                pair_scores = score_function(query_embeddings, sub_corpus_embeddings)

                # Get top-k values
                pair_scores_top_k_values, pair_scores_top_k_idx = torch.topk(
                    pair_scores,
                    min(max_k, len(pair_scores[0])),
                    dim=1,
                    largest=True,
                    sorted=False,
                )
                pair_scores_top_k_values = pair_scores_top_k_values.cpu().tolist()
                pair_scores_top_k_idx = pair_scores_top_k_idx.cpu().tolist()

                for query_itr in range(len(query_embeddings)):
                    for sub_corpus_id, score in zip(
                        pair_scores_top_k_idx[query_itr],
                        pair_scores_top_k_values[query_itr],
                    ):
                        corpus_id = self.corpus_ids[corpus_start_idx + sub_corpus_id]
                        queries_result_list[name][query_itr].append({"corpus_id": corpus_id, "score": score})

        # Compute scores
        logger.info("Computing metrics...")
        scores = {name: self._compute_metrics(queries_result_list[name]) for name in self.score_functions}

        return scores

    def _compute_metrics(self, queries_result_list: List[object]):
        """
        Compute metrics for a list of queries

        Args:
            queries_result_list (`List[List[Dict]]`): List of lists of dictionaries with keys "corpus_id" and "score"

        Returns:
            `Dict[str, Dict[str, float]]`: Dictionary with keys "mrr@k", "ndcg@k", "accuracy@k", "precision_recall@k", "map@k"
                which values are dictionaries with scores for different k values
        """
        # Init score computation values
        num_hits_at_k = {"accuracy_at_" + str(k): 0 for k in self.accuracy_at_k}
        precisions_at_k = {"precision_at_" + str(k): [] for k in self.precision_recall_at_k}
        recall_at_k = {"recall_at_" + str(k): [] for k in self.precision_recall_at_k}
        MRR = {"mrr_at_" + str(k): 0 for k in self.mrr_at_k}
        ndcg = {"ndcg_at_" + str(k): [] for k in self.ndcg_at_k}
        AveP_at_k = {"map_at_" + str(k): [] for k in self.map_at_k}

        # Compute scores on results
        for query_itr in range(len(queries_result_list)):
            query_id = self.queries_ids[query_itr]

            # Sort scores
            top_hits = sorted(queries_result_list[query_itr], key=lambda x: x["score"], reverse=True)
            query_relevant_docs = self.relevant_docs[query_id]

            # Accuracy@k - We count the result correct, if at least one relevant doc is accross the top-k documents
            for k_val in self.accuracy_at_k:
                for hit in top_hits[0:k_val]:
                    if hit["corpus_id"] in query_relevant_docs:
                        num_hits_at_k["accuracy_at_" + str(k_val)] += 1
                        break

            # Precision and Recall@k
            for k_val in self.precision_recall_at_k:
                num_correct = 0
                for hit in top_hits[0:k_val]:
                    if hit["corpus_id"] in query_relevant_docs:
                        num_correct += 1

                precisions_at_k["precision_at_" + str(k_val)].append(num_correct / k_val)
                recall_at_k["recall_at_" + str(k_val)].append(num_correct / len(query_relevant_docs))

            # MRR@k
            for k_val in self.mrr_at_k:
                for rank, hit in enumerate(top_hits[0:k_val]):
                    if hit["corpus_id"] in query_relevant_docs:
                        MRR["mrr_at_" + str(k_val)] += 1.0 / (rank + 1)
                        break

            # NDCG@k
            for k_val in self.ndcg_at_k:
                predicted_relevance = [
                    1 if top_hit["corpus_id"] in query_relevant_docs else 0 for top_hit in top_hits[0:k_val]
                ]
                true_relevances = [1] * len(query_relevant_docs)

                ndcg_value = self.compute_dcg_at_k(predicted_relevance, k_val) / self.compute_dcg_at_k(
                    true_relevances, k_val
                )
                ndcg["ndcg_at_" + str(k_val)].append(ndcg_value)

            # MAP@k
            for k_val in self.map_at_k:
                num_correct = 0
                sum_precisions = 0

                for rank, hit in enumerate(top_hits[0:k_val]):
                    if hit["corpus_id"] in query_relevant_docs:
                        num_correct += 1
                        sum_precisions += num_correct / (rank + 1)

                avg_precision = sum_precisions / min(k_val, len(query_relevant_docs))
                AveP_at_k["map_at_" + str(k_val)].append(avg_precision)

        # Compute averages
        for k in num_hits_at_k:
            num_hits_at_k[k] /= len(self.queries)

        for k in precisions_at_k:
            precisions_at_k[k] = np.mean(precisions_at_k[k])

        for k in recall_at_k:
            recall_at_k[k] = np.mean(recall_at_k[k])

        for k in ndcg:
            ndcg[k] = np.mean(ndcg[k])

        for k in MRR:
            MRR[k] /= len(self.queries)

        for k in AveP_at_k:
            AveP_at_k[k] = np.mean(AveP_at_k[k])

        return {**num_hits_at_k, **precisions_at_k, **recall_at_k, **MRR, **ndcg, **AveP_at_k}

    @staticmethod
    def compute_dcg_at_k(relevances, k):
        dcg = 0
        for i in range(min(len(relevances), k)):
            dcg += relevances[i] / np.log2(i + 2)  # +2 as we start our idx at 0
        return dcg



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class Touche2020(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "Touche2020",
            "beir_name": "webis-touche2020",
            "description": "Touché Task 1: Argument Retrieval for Controversial Questions",
            "reference": "https://webis.de/events/touche-20/shared-task-1.html",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class MedrxivClusteringS2S(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "MedrxivClusteringS2S",
            "hf_hub_name": "mteb/medrxiv-clustering-s2s",
            "description": "Clustering of titles from medrxiv. Clustering of 10 sets, based on the main category.",
            "reference": "https://api.biorxiv.org/",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "35191c8c0dca72d8ff3efcd72aa802307d469663",
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class BlurbsClusteringS2S(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "BlurbsClusteringS2S",
            "hf_hub_name": "slvnwhrl/blurbs-clustering-s2s",
            "description": "Clustering of book titles. Clustering of 28 sets, either on the main or secondary genre.",
            "reference": "https://www.inf.uni-hamburg.de/en/inst/ab/lt/resources/data/germeval-2019-hmc.html",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["de"],
            "main_score": "v_measure",
            "revision": "9bfff9a7f8f6dc6ffc9da71c48dd48b68696471d",
        }



import logging
from collections import defaultdict

from ..evaluation.evaluators import PairClassificationEvaluator
from .AbsTask import AbsTask


class AbsTaskPairClassification(AbsTask):
    """
    Abstract class for PairClassificationTasks
    The similarity is computed between pairs and the results are ranked. Average precision
    is computed to measure how well the methods can be used for pairwise pair classification.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def evaluate(self, model, split="test", **kwargs):
        if not self.data_loaded:
            self.load_data()

        data_split = self.dataset[split][0]

        logging.getLogger("sentence_transformers.evaluation.PairClassificationEvaluator").setLevel(logging.WARN)
        evaluator = PairClassificationEvaluator(
            data_split["sent1"], data_split["sent2"], data_split["labels"], **kwargs
        )
        scores = evaluator.compute_metrics(model)

        # Compute max
        max_scores = defaultdict(list)
        for sim_fct in scores:
            for metric in ["accuracy", "f1", "ap"]:
                max_scores[metric].append(scores[sim_fct][metric])

        for metric in max_scores:
            max_scores[metric] = max(max_scores[metric])

        scores["max"] = dict(max_scores)

        return scores



from ...abstasks import AbsTaskClassification


class ImdbClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "ImdbClassification",
            "hf_hub_name": "mteb/imdb",
            "description": "Large Movie Review Dataset",
            "reference": "http://www.aclweb.org/anthology/P11-1015",
            "category": "p2p",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "accuracy",
            "revision": "3d86128a09e091d6018b6d26cad27f2739fc2db7",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class FEVER(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "FEVER",
            "beir_name": "fever",
            "description": (
                "FEVER (Fact Extraction and VERification) consists of 185,445 claims generated by altering sentences"
                " extracted from Wikipedia and subsequently verified without knowledge of the sentence they were"
                " derived from."
            ),
            "reference": "https://fever.ai/",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks import AbsTaskClassification, MultilingualTask

_LANGUAGES = ["en", "de", "es", "fr", "hi", "th"]


class MTOPDomainClassification(MultilingualTask, AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "MTOPDomainClassification",
            "hf_hub_name": "mteb/mtop_domain",
            "description": "MTOP: Multilingual Task-Oriented Semantic Parsing",
            "reference": "https://arxiv.org/pdf/2008.09335.pdf",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["validation", "test"],
            "eval_langs": _LANGUAGES,
            "main_score": "accuracy",
            "revision": "d80d48c1eb48d3562165c59d59d0034df9fff0bf",
        }



import numpy as np
import tqdm

from ..evaluation.evaluators import ClusteringEvaluator
from .AbsTask import AbsTask


class AbsTaskClustering(AbsTask):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def evaluate(self, model, split="test", **kwargs):
        if not self.data_loaded:
            self.load_data()

        v_measures = []
        for cluster_set in tqdm.tqdm(self.dataset[split], desc="Clustering"):
            evaluator = ClusteringEvaluator(cluster_set["sentences"], cluster_set["labels"], **kwargs)
            metrics = evaluator(model)
            v_measures.append(metrics["v_measure"])

        v_mean = np.mean(v_measures)
        v_std = np.std(v_measures)
        return {"v_measure": v_mean, "v_measure_std": v_std}



from ...abstasks.AbsTaskPairClassification import AbsTaskPairClassification


class Ocnli(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            'name': 'Ocnli',
            "hf_hub_name": "C-MTEB/OCNLI",
            'description': 'Original Chinese Natural Language Inference dataset',
            "reference": "https://arxiv.org/abs/2010.05444",
            'category': 's2s',
            'type': 'PairClassification',
            'eval_splits': ['validation'],
            'eval_langs': ['zh'],
            'main_score': 'ap',
        }


class Cmnli(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            'name': 'Cmnli',
            "hf_hub_name": "C-MTEB/CMNLI",
            'description': 'Chinese Multi-Genre NLI',
            "reference": "https://huggingface.co/datasets/clue/viewer/cmnli",
            'category': 's2s',
            'type': 'PairClassification',
            'eval_splits': ['validation'],
            'eval_langs': ['zh'],
            'main_score': 'ap',
        }



import datasets

from mteb.abstasks import AbsTaskBitextMining


class BornholmBitextMining(AbsTaskBitextMining):
    @property
    def description(self):
        return {
            "name": "BornholmBitextMining",
            "hf_hub_name": "strombergnlp/bornholmsk_parallel",
            "description": "Danish Bornholmsk Parallel Corpus. "
            + "Bornholmsk is a Danish dialect spoken on the island of Bornholm, Denmark. "
            + "Historically it is a part of east Danish which was also spoken in Scania and Halland, Sweden.",
            "reference": "https://aclanthology.org/W19-6138/",
            "type": "BitextMining",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["da", "da-bornholm"],
            "main_score": "f1",
            "revision": "3bc5cfb4ec514264fe2db5615fac9016f7251552",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub and convert it to the standard format.
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"],
            revision=self.description.get("revision", None),
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        # Convert to standard format
        self.dataset = self.dataset.rename_column("da", "sentence1")
        self.dataset = self.dataset.rename_column("da_bornholm", "sentence2")



from ..evaluation.evaluators import RerankingEvaluator
from .AbsTask import AbsTask


class AbsTaskReranking(AbsTask):
    """
    Abstract class for re-ranking experiments.
    Child-classes must implement the following properties:
    self.corpus = {'dev': Dict[id, str], 'test': Dict[id, str]}         #id => sentence
    self.queries = {'dev': Dict[id, str], 'test': Dict[id, str]}
    self.relevant_docs = {'dev': Dict[id, set], 'test': Dict[id, set]}
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def evaluate(self, model, split="test", **kwargs):
        if not self.data_loaded:
            self.load_data()

        data_split = self.dataset[split]

        evaluator = RerankingEvaluator(data_split, **kwargs)
        scores = evaluator(model)

        return dict(scores)



from ...abstasks.AbsTaskPairClassification import AbsTaskPairClassification


class TwitterSemEval2015PC(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            "name": "TwitterSemEval2015",
            "hf_hub_name": "mteb/twittersemeval2015-pairclassification",
            "description": "Paraphrase-Pairs of Tweets from the SemEval 2015 workshop.",
            "reference": "https://alt.qcri.org/semeval2015/task1/",
            "category": "s2s",
            "type": "PairClassification",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ap",
            "revision": "70970daeab8776df92f5ea462b6173c0b46fd2d1",
        }



from ...abstasks.AbsTaskReranking import AbsTaskReranking


class SciDocsReranking(AbsTaskReranking):
    @property
    def description(self):
        return {
            "name": "SciDocsRR",
            "hf_hub_name": "mteb/scidocs-reranking",
            "description": "Ranking of related scientific papers based on their title.",
            "reference": "https://allenai.org/data/scidocs",
            "type": "Reranking",
            "category": "s2s",
            "eval_splits": ["test", "validation"],
            "eval_langs": ["en"],
            "main_score": "map",
            "revision": "d3c5e1fc0b855ab6097bf1cda04dd73947d7caab",
        }



from ...abstasks import AbsTaskClassification

class TNews(AbsTaskClassification):
    @property
    def description(self):
        return {
            'name': 'TNews',
            'hf_hub_name': 'C-MTEB/TNews-classification',
            'description': 'Short Text Classification for News',
            "reference": "https://www.cluebenchmarks.com/introduce.html",
            'type': 'Classification',
            'category': 's2s',
            'eval_splits': ['validation'],
            'eval_langs': ['zh'],
            'main_score': 'accuracy',
            'samples_per_label': 32,
        }


class IFlyTek(AbsTaskClassification):
    @property
    def description(self):
        return {
            'name': 'IFlyTek',
            'hf_hub_name': 'C-MTEB/IFlyTek-classification',
            'description': 'Long Text classification for the description of Apps',
            "reference": "https://www.cluebenchmarks.com/introduce.html",
            'type': 'Classification',
            'category': 's2s',
            'eval_splits': ['validation'],
            'eval_langs': ['zh'],
            'main_score': 'accuracy',
            'samples_per_label': 32,
            'n_experiments': 5
        }


class MultilingualSentiment(AbsTaskClassification):
    @property
    def description(self):
        return {
            'name': 'MultilingualSentiment',
            'hf_hub_name': 'C-MTEB/MultilingualSentiment-classification',
            'description': 'A collection of multilingual sentiments datasets grouped into 3 classes -- positive, neutral, negative',
            "reference": "https://github.com/tyqiangz/multilingual-sentiment-datasets",
            'category': 's2s',
            'type': 'Classification',
            'eval_splits': ['validation'],
            'eval_langs': ['zh'],
            'main_score': 'accuracy',
            'samples_per_label': 32,
        }



class JDReview(AbsTaskClassification):
    @property
    def description(self):
        return {
            'name': 'JDReview',
            'hf_hub_name': 'C-MTEB/JDReview-classification',
            'description': 'review for iphone',
            'category': 's2s',
            'type': 'Classification',
            'eval_splits': ['test'],
            'eval_langs': ['zh'],
            'main_score': 'accuracy',
            'samples_per_label': 32,
        }


class OnlineShopping(AbsTaskClassification):
    @property
    def description(self):
        return {
            'name': 'OnlineShopping',
            'hf_hub_name': 'C-MTEB/OnlineShopping-classification',
            'description': 'Sentiment Analysis of User Reviews on Online Shopping Websites',
            'category': 's2s',
            'type': 'Classification',
            'eval_splits': ['test'],
            'eval_langs': ['zh'],
            'main_score': 'accuracy',
            'samples_per_label': 32,
        }


class Waimai(AbsTaskClassification):
    @property
    def description(self):
        return {
            'name': 'Waimai',
            'hf_hub_name': 'C-MTEB/waimai-classification',
            'description': 'Sentiment Analysis of user reviews on takeaway platforms',
            'category': 's2s',
            'type': 'Classification',
            'eval_splits': ['test'],
            'eval_langs': ['zh'],
            'main_score': 'accuracy',
            'samples_per_label': 32,
        }


from ...abstasks import AbsTaskBitextMining, CrosslingualTask

_LANGUAGES = ["de-en", "fr-en", "ru-en", "zh-en"]


class BUCCBitextMining(AbsTaskBitextMining, CrosslingualTask):
    @property
    def description(self):
        return {
            "name": "BUCC",
            "hf_hub_name": "mteb/bucc-bitext-mining",
            "description": "BUCC bitext mining dataset",
            "reference": "https://comparable.limsi.fr/bucc2018/bucc2018-task.html",
            "type": "BitextMining",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": _LANGUAGES,
            "main_score": "f1",
            "revision": "d51519689f32196a32af33b075a01d0e7c51e252",
        }



from mteb.abstasks.AbsTaskClassification import AbsTaskClassification


class AngryTweetsClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "AngryTweetsClassification",
            "hf_hub_name": "DDSC/angry-tweets",
            "description": "A sentiment dataset with 3 classes (positiv, negativ, neutral) for Danish tweets",
            "reference": "https://aclanthology.org/2021.nodalida-main.53/",
            "eval_splits": ["test"],
            "eval_langs": ["da"],
            "type": "Classification",
            "category": "s2s",
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "20b0e6081892e78179356fada741b7afa381443d",
        }



import logging

from ..evaluation.evaluators import STSEvaluator
from .AbsTask import AbsTask

logger = logging.getLogger(__name__)

class AbsTaskSTS(AbsTask):
    """
    Abstract class for re-ranking experiments.
    Child-classes must implement the following properties:
    self.corpus = {'dev': Dict[id, str], 'test': Dict[id, str]}         #id => sentence
    self.queries = {'dev': Dict[id, str], 'test': Dict[id, str]}
    self.relevant_docs = {'dev': Dict[id, set], 'test': Dict[id, set]}
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def min_score(self):
        return self.description["min_score"]

    @property
    def max_score(self):
        return self.description["max_score"]

    def evaluate(self, model, split, **kwargs):
        if not self.data_loaded:
            self.load_data()

        if self.is_crosslingual:
            scores = {}
            for lang in self.dataset:
                logger.info(f"Task: {self.description['name']}, split: {split}, language: {lang}. Running...")
                data_split = self.dataset[lang][split]
                scores[lang] = self._evaluate_split(model, data_split, **kwargs)
        else:
            logger.info(f"\nTask: {self.description['name']}, split: {split}. Running...")
            data_split = self.dataset[split]
            scores = self._evaluate_split(model, data_split, **kwargs)

        return scores

    def _evaluate_split(self, model, data_split, **kwargs):
        def normalize(x):
            return (x - self.min_score) / (self.max_score - self.min_score)

        normalized_scores = list(map(normalize, data_split["score"]))
        evaluator = STSEvaluator(data_split["sentence1"], data_split["sentence2"], normalized_scores, **kwargs)
        metrics = evaluator(model)
        return metrics



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class BiorxivClusteringS2S(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "BiorxivClusteringS2S",
            "hf_hub_name": "mteb/biorxiv-clustering-s2s",
            "description": "Clustering of titles from biorxiv. Clustering of 10 sets, based on the main category.",
            "reference": "https://api.biorxiv.org/",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "258694dd0231531bc1fd9de6ceb52a0853c6d908",
        }



import logging
from collections import defaultdict

import numpy as np

from ..evaluation.evaluators import (
    kNNClassificationEvaluator,
    kNNClassificationEvaluatorPytorch,
    logRegClassificationEvaluator,
)
from .AbsTask import AbsTask

logger = logging.getLogger(__name__)


class AbsTaskClassification(AbsTask):
    """
    Abstract class for kNN classification tasks
    The similarity is computed between pairs and the results are ranked. Average precision
    is computed to measure how well the methods can be used for classification. #TODO:
    """

    def __init__(self, method="logReg", n_experiments=None, samples_per_label=None, k=3, batch_size=32, **kwargs):
        super().__init__(**kwargs)
        self.batch_size = batch_size
        self.method = method

        # Bootstrap parameters
        self.n_experiments = n_experiments if n_experiments is not None else self.description.get("n_experiments", 10)
        self.samples_per_label = (
            samples_per_label if samples_per_label is not None else self.description.get("samples_per_label", 8)
        )

        # kNN parameters
        self.k = k

    def _add_main_score(self, scores):
        if self.description["main_score"] in scores:
            scores["main_score"] = scores[self.description["main_score"]]
        else:
            logger.warn(f"main score {self.description['main_score']} not found in scores {scores.keys()}")

    def evaluate(self, model, eval_split="test", train_split="train", **kwargs):
        if not self.data_loaded:
            self.load_data()

        if self.is_multilingual:
            scores = {}
            for lang in self.dataset:
                logger.info(f"\nTask: {self.description['name']}, split: {eval_split}, language: {lang}. Running...")
                scores[lang] = self._evaluate_monolingual(model, self.dataset[lang], eval_split, train_split, **kwargs)
                self._add_main_score(scores[lang])
        else:
            logger.info(f"\nTask: {self.description['name']}, split: {eval_split}. Running...")
            scores = self._evaluate_monolingual(model, self.dataset, eval_split, train_split, **kwargs)
            self._add_main_score(scores)

        return scores

    def _evaluate_monolingual(self, model, dataset, eval_split="test", train_split="train", **kwargs):
        train_split = dataset[train_split]
        eval_split = dataset[eval_split]
        params = {"k": self.k, "batch_size": self.batch_size}
        params.update(kwargs)

        scores = []
        test_cache, idxs = None, None  # we store idxs to make the shuffling reproducible
        for i in range(self.n_experiments):
            logger.info("=" * 10 + f" Experiment {i+1}/{self.n_experiments} " + "=" * 10)
            # Bootstrap `self.samples_per_label` samples per label for each split
            X_sampled, y_sampled, idxs = self._undersample_data(
                train_split["text"], train_split["label"], self.samples_per_label, idxs
            )

            if self.method == "kNN":
                evaluator = kNNClassificationEvaluator(
                    X_sampled, y_sampled, eval_split["text"], eval_split["label"], **params
                )
            elif self.method == "kNN-pytorch":
                evaluator = kNNClassificationEvaluatorPytorch(
                    X_sampled, y_sampled, eval_split["text"], eval_split["label"], **params
                )
            elif self.method == "logReg":
                evaluator = logRegClassificationEvaluator(
                    X_sampled, y_sampled, eval_split["text"], eval_split["label"], **params
                )
            else:
                raise ValueError(f"Method {self.method} not supported")

            scores_exp, test_cache = evaluator(model, test_cache=test_cache)
            scores.append(scores_exp)

        if self.n_experiments == 1:
            return scores[0]
        else:
            avg_scores = {k: np.mean([s[k] for s in scores]) for k in scores[0].keys()}
            std_errors = {k + "_stderr": np.std([s[k] for s in scores]) for k in scores[0].keys()}
            return {**avg_scores, **std_errors}

    def _undersample_data(self, X, y, samples_per_label, idxs=None):
        """Undersample data to have samples_per_label samples of each label"""
        X_sampled = []
        y_sampled = []
        if idxs is None:
            idxs = np.arange(len(y))
        np.random.shuffle(idxs)
        label_counter = defaultdict(int)
        for i in idxs:
            if label_counter[y[i]] < samples_per_label:
                X_sampled.append(X[i])
                y_sampled.append(y[i])
                label_counter[y[i]] += 1
        return X_sampled, y_sampled, idxs



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class ATEC(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "ATEC",
            "hf_hub_name": "C-MTEB/ATEC",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 1,
        }



class BQ(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "BQ",
            "hf_hub_name": "C-MTEB/BQ",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 1,
        }


class LCQMC(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "LCQMC",
            "hf_hub_name": "C-MTEB/LCQMC",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 1,
        }



class PAWSX(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "PAWSX",
            "hf_hub_name": "C-MTEB/PAWSX",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 1,
        }


class STSB(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "STSB",
            "hf_hub_name": "C-MTEB/STSB",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
        }


class AFQMC(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "AFQMC",
            "hf_hub_name": "C-MTEB/AFQMC",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["validation"],
            "eval_langs": ["zh"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 1,
        }



class QBQTC(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "QBQTC",
            "hf_hub_name": "C-MTEB/QBQTC",
            "reference": "https://github.com/CLUEbenchmark/QBQTC/tree/main/dataset",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["zh"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 1,
        }



from ...abstasks import AbsTaskClassification


class CbdClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "CBD",
            "hf_hub_name": "PL-MTEB/cbd",
            "description": "Polish Tweets annotated for cyberbullying detection.",
            "reference": "http://2019.poleval.pl/files/poleval2019.pdf",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "accuracy"
        }


class PolEmo2InClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "PolEmo2.0-IN",
            "hf_hub_name": "PL-MTEB/polemo2_in",
            "description": "A collection of Polish online reviews from four domains: medicine, hotels, products and "
                           "school. The PolEmo2.0-IN task is to predict the sentiment of in-domain (medicine and hotels) "
                           "reviews.",
            "reference": "https://aclanthology.org/K19-1092.pdf",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "accuracy"
        }


class PolEmo2OutClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "PolEmo2.0-OUT",
            "hf_hub_name": "PL-MTEB/polemo2_out",
            "description": "A collection of Polish online reviews from four domains: medicine, hotels, products and "
                           "school. The PolEmo2.0-OUT task is to predict the sentiment of out-of-domain (products and "
                           "school) reviews using models train on reviews from medicine and hotels domains.",
            "reference": "https://aclanthology.org/K19-1092.pdf",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "accuracy"
        }


class AllegroReviewsClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "AllegroReviews",
            "hf_hub_name": "PL-MTEB/allegro-reviews",
            "description": "A Polish dataset for sentiment classification on reviews from e-commerce marketplace Allegro.",
            "reference": "https://aclanthology.org/2020.acl-main.111.pdf",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "accuracy"
        }


class PacClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "PAC",
            "hf_hub_name": "laugustyniak/abusive-clauses-pl",
            "description": "Polish Abusive Clauses Dataset",
            "reference": "https://arxiv.org/pdf/2211.13112.pdf",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "accuracy"
        }



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class SickrSTS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "SICK-R",
            "hf_hub_name": "mteb/sickr-sts",
            "description": "Semantic Textual Similarity SICK-R dataset as described here:",
            "reference": "https://www.aclweb.org/anthology/S14-2001.pdf",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 1,
            "max_score": 5,
            "revision": "a6ea5a8cab320b040a23452cc28066d9beae2cee",
        }



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class STSBenchmarkSTS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "STSBenchmark",
            "hf_hub_name": "mteb/stsbenchmark-sts",
            "description": "Semantic Textual Similarity Benchmark (STSbenchmark) dataset.",
            "reference": "http://ixa2.si.ehu.es/stswiki/index.php/STSbenchmark",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["validation", "test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
            "revision": "b0fddb56ed78048fa8b90373c8a3cfc37b684831",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class ClimateFEVER(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "ClimateFEVER",
            "beir_name": "climate-fever",
            "description": (
                "CLIMATE-FEVER is a dataset adopting the FEVER methodology that consists of 1,535 real-world claims"
                " regarding climate-change."
            ),
            "reference": "https://www.sustainablefinance.uzh.ch/en/research/climate-fever.html",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class BiorxivClusteringP2P(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "BiorxivClusteringP2P",
            "hf_hub_name": "mteb/biorxiv-clustering-p2p",
            "description": (
                "Clustering of titles+abstract from biorxiv. Clustering of 10 sets, based on the main category."
            ),
            "reference": "https://api.biorxiv.org/",
            "type": "Clustering",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "65b79d1d13f80053f67aca9498d9402c2d9f1f40",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class SCIDOCS(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "SCIDOCS",
            "beir_name": "scidocs",
            "description": (
                "SciDocs, a new evaluation benchmark consisting of seven document-level tasks ranging from citation"
                " prediction, to document classification and recommendation."
            ),
            "reference": "https://allenai.org/data/scidocs",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks import AbsTaskBitextMining, CrosslingualTask

_LANGUAGES = [
    "sqi-eng",
    "fry-eng",
    "kur-eng",
    "tur-eng",
    "deu-eng",
    "nld-eng",
    "ron-eng",
    "ang-eng",
    "ido-eng",
    "jav-eng",
    "isl-eng",
    "slv-eng",
    "cym-eng",
    "kaz-eng",
    "est-eng",
    "heb-eng",
    "gla-eng",
    "mar-eng",
    "lat-eng",
    "bel-eng",
    "pms-eng",
    "gle-eng",
    "pes-eng",
    "nob-eng",
    "bul-eng",
    "cbk-eng",
    "hun-eng",
    "uig-eng",
    "rus-eng",
    "spa-eng",
    "hye-eng",
    "tel-eng",
    "afr-eng",
    "mon-eng",
    "arz-eng",
    "hrv-eng",
    "nov-eng",
    "gsw-eng",
    "nds-eng",
    "ukr-eng",
    "uzb-eng",
    "lit-eng",
    "ina-eng",
    "lfn-eng",
    "zsm-eng",
    "ita-eng",
    "cmn-eng",
    "lvs-eng",
    "glg-eng",
    "ceb-eng",
    "bre-eng",
    "ben-eng",
    "swg-eng",
    "arq-eng",
    "kab-eng",
    "fra-eng",
    "por-eng",
    "tat-eng",
    "oci-eng",
    "pol-eng",
    "war-eng",
    "aze-eng",
    "vie-eng",
    "nno-eng",
    "cha-eng",
    "mhr-eng",
    "dan-eng",
    "ell-eng",
    "amh-eng",
    "pam-eng",
    "hsb-eng",
    "srp-eng",
    "epo-eng",
    "kzj-eng",
    "awa-eng",
    "fao-eng",
    "mal-eng",
    "ile-eng",
    "bos-eng",
    "cor-eng",
    "cat-eng",
    "eus-eng",
    "yue-eng",
    "swe-eng",
    "dtp-eng",
    "kat-eng",
    "jpn-eng",
    "csb-eng",
    "xho-eng",
    "orv-eng",
    "ind-eng",
    "tuk-eng",
    "max-eng",
    "swh-eng",
    "hin-eng",
    "dsb-eng",
    "ber-eng",
    "tam-eng",
    "slk-eng",
    "tgl-eng",
    "ast-eng",
    "mkd-eng",
    "khm-eng",
    "ces-eng",
    "tzl-eng",
    "urd-eng",
    "ara-eng",
    "kor-eng",
    "yid-eng",
    "fin-eng",
    "tha-eng",
    "wuu-eng",
]


class TatoebaBitextMining(AbsTaskBitextMining, CrosslingualTask):
    @property
    def description(self):
        return {
            "name": "Tatoeba",
            "hf_hub_name": "mteb/tatoeba-bitext-mining",
            "description": "1,000 English-aligned sentence pairs for each language based on the Tatoeba corpus",
            "reference": "https://github.com/facebookresearch/LASER/tree/main/data/tatoeba/v1",
            "type": "BitextMining",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": _LANGUAGES,
            "main_score": "f1",
            "revision": "9080400076fbadbb4c4dcb136ff4eddc40b42553",
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class ArxivClusteringS2S(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "ArxivClusteringS2S",
            "hf_hub_name": "mteb/arxiv-clustering-s2s",
            "description": (
                "Clustering of titles from arxiv. Clustering of 30 sets, either on the main or secondary category"
            ),
            "reference": "https://www.kaggle.com/Cornell-University/arxiv",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "f910caf1a6075f7329cdf8c1a6135696f37dbd53",
        }



from ...abstasks import AbsTaskClassification


class Banking77Classification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "Banking77Classification",
            "hf_hub_name": "mteb/banking77",
            "description": "Dataset composed of online banking queries annotated with their corresponding intents.",
            "reference": "https://arxiv.org/abs/2003.04807",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "accuracy",
            "revision": "0fd18e25b25c072e09e0d92ab615fda904d66300",
        }



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class STS15STS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "STS15",
            "hf_hub_name": "mteb/sts15-sts",
            "description": "SemEval STS 2015 dataset",
            "reference": "http://alt.qcri.org/semeval2015/task2/",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
            "revision": "ae752c7c21bf194d8b67fd573edf7ae58183cbe3",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class MSMARCO(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "MSMARCO",
            "beir_name": "msmarco",
            "description": "MS MARCO is a collection of datasets focused on deep learning in search",
            "reference": "https://microsoft.github.io/msmarco/",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["dev", "test"],  # "validation" if using latest BEIR i.e. HFDataLoader
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks.AbsTaskReranking import AbsTaskReranking


class AskUbuntuDupQuestions(AbsTaskReranking):
    @property
    def description(self):
        return {
            "name": "AskUbuntuDupQuestions",
            "hf_hub_name": "mteb/askubuntudupquestions-reranking",
            "description": (
                "AskUbuntu Question Dataset - Questions from AskUbuntu with manual annotations marking pairs of"
                " questions as similar or non-similar"
            ),
            "reference": "https://github.com/taolei87/askubuntu",
            "type": "Reranking",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "map",
            "revision": "2000358ca161889fa9c082cb41daa8dcfb161a54",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class ArguAna(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "ArguAna",
            "beir_name": "arguana",
            "description": "NFCorpus: A Full-Text Learning to Rank Dataset for Medical Information Retrieval",
            "reference": "http://argumentation.bplaced.net/arguana/data",
            "type": "Retrieval",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



from ...abstasks import AbsTaskClassification


class TweetSentimentExtractionClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "TweetSentimentExtractionClassification",
            "hf_hub_name": "mteb/tweet_sentiment_extraction",
            "description": "",
            "reference": "https://www.kaggle.com/competitions/tweet-sentiment-extraction/overview",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 32,
            "revision": "d604517c81ca91fe16a244d1248fc021f9ecee7a",
        }



from mteb.abstasks import AbsTaskClassification


class SweRecClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "SweRecClassification",
            "hf_hub_name": "ScandEval/swerec-mini",  # using the mini version to keep results ~comparable to ScandEval
            "description": "A Swedish dataset for sentiment classification on review",
            "reference": "https://aclanthology.org/2023.nodalida-1.20/",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["sv"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "3c62f26bafdc4c4e1c16401ad4b32f0a94b46612",
        }



from ...abstasks.AbsTaskPairClassification import AbsTaskPairClassification


class SickePLPC(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            "name": "SICK-E-PL",
            "hf_hub_name": "PL-MTEB/sicke-pl-pairclassification",
            "description": "Polish version of SICK dataset for textual entailment.",
            "reference": "https://aclanthology.org/2020.lrec-1.207.pdf",
            "category": "s2s",
            "type": "PairClassification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "ap",
        }


class PpcPC(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            "name": "PPC",
            "hf_hub_name": "PL-MTEB/ppc-pairclassification",
            "description": "Polish Paraphrase Corpus",
            "reference": "https://arxiv.org/pdf/2207.12759.pdf",
            "category": "s2s",
            "type": "PairClassification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "ap"
        }


class CdscePC(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            "name": "CDSC-E",
            "hf_hub_name": "PL-MTEB/cdsce-pairclassification",
            "description": "Compositional Distributional Semantics Corpus for textual entailment.",
            "reference": "https://aclanthology.org/P17-1073.pdf",
            "category": "s2s",
            "type": "PairClassification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "ap"
        }


class PscPC(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            "name": "PSC",
            "hf_hub_name": "PL-MTEB/psc-pairclassification",
            "description": "Polish Summaries Corpus",
            "reference": "http://www.lrec-conf.org/proceedings/lrec2014/pdf/1211_Paper.pdf",
            "category": "s2s",
            "type": "PairClassification",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "ap"
        }



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class STS13STS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "STS13",
            "hf_hub_name": "mteb/sts13-sts",
            "description": "SemEval STS 2013 dataset.",
            "reference": "https://www.aclweb.org/anthology/S13-1004/",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
            "revision": "7e90230a92c190f1bf69ae9002b8cea547a64cca",
        }



# SuperLIM tasks
import datasets

from mteb.abstasks import AbsTaskClassification


class DalajClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "DalajClassification",
            "hf_hub_name": "AI-Sweden/SuperLim",
            "description": "A Swedish dataset for linguistic acceptability. Available as a part of Superlim.",
            "reference": "https://spraakbanken.gu.se/en/resources/superlim",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["sv"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "7ebf0b4caa7b2ae39698a889de782c09e6f5ee56",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"],
            "dalaj",  # chose the relevant subset
            revision=self.description.get("revision"),
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        """
        This dataset consist of two columns of relevance, "original_sentence" and "corrected_sentence".
        We will use the original sentence as we "wrong" sentence and the corrected sentence as the "correct" sentence
        """

        def __convert_sample_to_classification(sample):
            text = sample["original_sentence"] + sample["corrected_sentence"]
            label = [1] * len(sample["original_sentence"]) + [0] * len(sample["corrected_sentence"])
            return {"text": text, "label": label}

        columns_to_keep = ["original_sentence", "corrected_sentence"]
        for split in self.dataset:
            columns_names = self.dataset[split].column_names  # type: ignore
            columns_to_remove = [col for col in columns_names if col not in columns_to_keep]
            self.dataset[split] = self.dataset[split].remove_columns(columns_to_remove)  # type: ignore

        self.dataset = self.dataset.map(
            __convert_sample_to_classification, batched=True, remove_columns=columns_to_keep
        )



import datasets

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification


class DKHateClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "DKHateClassification",
            "hf_hub_name": "DDSC/dkhate",
            "description": "Danish Tweets annotated for Hate Speech either being Offensive or not",
            "reference": "https://aclanthology.org/2020.lrec-1.430/",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["da"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "59d12749a3c91a186063c7d729ec392fda94681c",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"], revision=self.description.get("revision", None)
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        # convert label to a 0/1 label
        labels = self.dataset["train"]["label"]  # type: ignore
        lab2idx = {lab: idx for idx, lab in enumerate(set(labels))}
        self.dataset = self.dataset.map(lambda x: {"label": lab2idx[x["label"]]}, remove_columns=["label"])



import os

import datasets

from .AbsTask import AbsTask
import logging

logger = logging.getLogger(__name__)

class BeIRTask(AbsTask):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def load_data(self, eval_splits=None, **kwargs):
        """
        Load dataset from BeIR benchmark. TODO: replace with HF hub once datasets are moved there
        """
        try:
            from beir import util
        except ImportError:
            raise Exception("Retrieval tasks require beir package. Please install it with `pip install mteb[beir]`")

        USE_HF_DATASETS = False

        # TODO @nouamane: move non-distributed to `HFDataLoader`
        if os.getenv("RANK", None) is not None:
            if self.description["beir_name"].startswith("cqadupstack"):
                raise ImportError("CQADupstack is incompatible with BEIR's HFDataLoader in a distributed setting")
            from beir.datasets.data_loader_hf import HFDataLoader 
            logger.info("Using HFDataLoader for BeIR")
            USE_HF_DATASETS = True
        else:
            from beir.datasets.data_loader import GenericDataLoader 
            logger.info("Using GenericDataLoader for BeIR")

        if self.data_loaded:
            return
        if eval_splits is None:
            eval_splits = self.description["eval_splits"]
        dataset = self.description["beir_name"]
        dataset, sub_dataset = dataset.split("/") if "cqadupstack" in dataset else (dataset, None)

        self.corpus, self.queries, self.relevant_docs = {}, {}, {}
        for split in eval_splits:
            if USE_HF_DATASETS:
                self.corpus[split], self.queries[split], self.relevant_docs[split] = HFDataLoader(
                    hf_repo=f"BeIR/{dataset}"
                ).load(split=split)
            else:
                url = f"https://public.ukp.informatik.tu-darmstadt.de/thakur/BEIR/datasets/{dataset}.zip"
                download_path = os.path.join(datasets.config.HF_DATASETS_CACHE, "BeIR")
                data_path = util.download_and_unzip(url, download_path)
                data_path = f"{data_path}/{sub_dataset}" if sub_dataset else data_path
                self.corpus[split], self.queries[split], self.relevant_docs[split] = GenericDataLoader(
                    data_folder=data_path
                ).load(split=split)
        self.data_loaded = True



from ...abstasks.AbsTaskReranking import AbsTaskReranking


class StackOverflowDupQuestions(AbsTaskReranking):
    @property
    def description(self):
        return {
            "name": "StackOverflowDupQuestions",
            "hf_hub_name": "mteb/stackoverflowdupquestions-reranking",
            "description": (
                "Stack Overflow Duplicate Questions Task for questions with the tags Java, JavaScript and Python"
            ),
            "reference": "https://www.microsoft.com/en-us/research/uploads/prod/2019/03/nl4se18LinkSO.pdf",
            "type": "Reranking",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "map",
            "revision": "e185fbe320c72810689fc5848eb6114e1ef5ec69",
        }



from mteb.abstasks import AbsTaskClassification


class LccSentimentClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "LccSentimentClassification",
            "hf_hub_name": "DDSC/lcc",
            "description": "The leipzig corpora collection, annotated for sentiment",
            "reference": "https://github.com/fnielsen/lcc-sentiment",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["da"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "de7ba3406ee55ea2cc52a0a41408fa6aede6d3c6",
        }



from ...abstasks import AbsTaskSTS, CrosslingualTask

_LANGUAGES = {
    "en": "en",
    "de": "de",
    "es": "es",
    "pl": "pl",
    "tr": "tr",
    "ar": "ar",
    "ru": "ru",
    "zh": "zh",
    "fr": "fr",
    "de-en": "de-en",
    "es-en": "es-en",
    "it": "it",
    "pl-en": "pl-en",
    "zh-en": "zh-en",
    "es-it": "es-it",
    "de-fr": "de-fr",
    "de-pl": "de-pl",
    "fr-pl": "fr-pl",
}


class STS22CrosslingualSTS(AbsTaskSTS, CrosslingualTask):
    @property
    def description(self):
        return {
            "name": "STS22",
            "hf_hub_name": "mteb/sts22-crosslingual-sts",
            "description": "SemEval 2022 Task 8: Multilingual News Article Similarity",
            "reference": "https://competitions.codalab.org/competitions/33835",
            "type": "STS",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": _LANGUAGES,
            "main_score": "cosine_spearman",
            "min_score": 1,
            "max_score": 4,
            "revision": "6d1ba47164174a496b7fa5d3569dae26a6813b80",
        }



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class ArxivClusteringP2P(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "ArxivClusteringP2P",
            "hf_hub_name": "mteb/arxiv-clustering-p2p",
            "description": (
                "Clustering of titles+abstract from arxiv. Clustering of 30 sets, either on the main or secondary"
                " category"
            ),
            "reference": "https://www.kaggle.com/Cornell-University/arxiv",
            "type": "Clustering",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "a122ad7f3f0291bf49cc6f4d32aa80929df69d5d",
        }



"""MTEB is an open library for benchmarking embeddings.
Note:
   VERSION needs to be formatted following the MAJOR.MINOR.PATCH convention
   (we need to follow this convention to be able to retrieve versioned scripts)
Inspired by: https://github.com/huggingface/datasets/blob/main/setup.py
To create the package for pypi.
0. Prerequisites:
   - Dependencies:
     - twine: "pip install twine"
     - wheel: "pip install wheel"
   - Create an account in (and join the 'datasets' project):
     - PyPI: https://pypi.org/
     - Test PyPI: https://test.pypi.org/
1. Change the version in:
   - mteb/__init__.py
   - setup.py
2. Commit these changes: "git commit -m 'Release: VERSION'"
3. Add a tag in git to mark the release: "git tag VERSION -m 'Add tag VERSION for pypi'"
   Push the tag to remote: git push --tags origin main
4. Build both the sources and the wheel. Do not change anything in setup.py between
   creating the wheel and the source distribution (obviously).
   First, delete any "build" directory that may exist from previous builds.
   For the wheel, run: "python setup.py bdist_wheel" in the top level directory.
   (this will build a wheel for the python version you use to build it).
   For the sources, run: "python setup.py sdist"
   You should now have a /dist directory with both .whl and .tar.gz source versions.
5. OPTIONAL: Check that everything looks correct by uploading the package to the pypi test server:
   twine upload dist/* -r pypitest --repository-url=https://test.pypi.org/legacy/
   Check that you can install it in a virtualenv/notebook by running:
   pip install huggingface_hub fsspec aiohttp
   pip install -U tqdm
   pip install -i https://testpypi.python.org/pypi datasets
6. Upload the final version to actual pypi:
   twine upload dist/* -r pypi
7. Fill release notes in the tag in github once everything is looking hunky-dory.
8. Change the version in __init__.py and setup.py to X.X.X+1.dev0 (e.g. VERSION=1.18.3 -> 1.18.4.dev0).
   Then push the change with a message 'set dev version'
"""


from setuptools import find_packages, setup


with open("README.md", mode="r", encoding="utf-8") as readme_file:
    readme = readme_file.read()


extras = {}
extras["beir"] = ["beir"]


setup(
    name="mteb",
    version="1.1.1",
    description="Massive Text Embedding Benchmark",
    long_description=readme,
    long_description_content_type="text/markdown",
    keywords="deep learning, text embeddings, benchmark",
    license="Apache",
    author="MTEB Contributors (https://github.com/embeddings-benchmark/mteb/graphs/contributors)",
    author_email="niklas@huggingface.co, nouamane@huggingface.co, info@nils-reimers.de",
    url="https://github.com/embeddings-benchmark/mteb",
    project_urls={
        "Huggingface Organization": "https://huggingface.co/mteb",
        "Source Code": "https://github.com/embeddings-benchmark/mteb",
    },
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "mteb=mteb.cmd:main",
        ]
    },
    python_requires=">=3.7.0",
    install_requires=[
        "datasets>=2.2.0",
        "jsonlines",
        "numpy",
        "requests>=2.26.0",
        "scikit_learn>=1.0.2",
        "scipy",
        "sentence_transformers>=2.2.0",
        "torch",
        "tqdm",
        "rich",
    ],
    extras_require=extras,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
)



from ...abstasks.AbsTaskPairClassification import AbsTaskPairClassification


class TwitterURLCorpusPC(AbsTaskPairClassification):
    @property
    def description(self):
        return {
            "name": "TwitterURLCorpus",
            "hf_hub_name": "mteb/twitterurlcorpus-pairclassification",
            "description": "Paraphrase-Pairs of Tweets.",
            "reference": "https://languagenet.github.io/",
            "category": "s2s",
            "type": "PairClassification",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ap",
            "revision": "8b6510b0b1fa4e4c4f879467980e9be563ec1cdf",
        }



import logging
from time import time
from typing import Dict, List

from sentence_transformers import SentenceTransformer
from sentence_transformers.models import Transformer, WordEmbeddings
import os

from .AbsTask import AbsTask

logger = logging.getLogger(__name__)

DRES_METHODS = ["encode_queries", "encode_corpus"]

class AbsTaskRetrieval(AbsTask):
    """
    Abstract class for re-ranking experiments.
    Child-classes must implement the following properties:
    self.corpus = Dict[id, Dict[str, str]] #id => dict with document datas like title and text
    self.queries = Dict[id, str] #id => query
    self.relevant_docs = List[id, id, score]
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @staticmethod
    def is_dres_compatible(model):
        for method in DRES_METHODS:
            op = getattr(model, method, None)
            if not (callable(op)):
                return False
        return True

    def evaluate(
        self,
        model,
        split="test",
        batch_size=128,
        corpus_chunk_size=None,
        score_function="cos_sim",
        **kwargs
    ):
        try:
            from beir.retrieval.evaluation import EvaluateRetrieval
        except ImportError:
            raise Exception("Retrieval tasks require beir package. Please install it with `pip install mteb[beir]`")

        if not self.data_loaded:
            self.load_data()

        corpus, queries, relevant_docs = self.corpus[split], self.queries[split], self.relevant_docs[split]
        model = model if self.is_dres_compatible(model) else DRESModel(model)

        if os.getenv("RANK", None) is None:
            # Non-distributed
            from beir.retrieval.search.dense import DenseRetrievalExactSearch as DRES
            model = DRES(
                model,
                batch_size=batch_size,
                corpus_chunk_size=corpus_chunk_size if corpus_chunk_size is not None else 50000,
                **kwargs,
            )
        
        else:
            # Distributed (multi-GPU)
            from beir.retrieval.search.dense import (
                DenseRetrievalParallelExactSearch as DRPES,
            )
            model = DRPES(
                model,
                batch_size=batch_size,
                corpus_chunk_size=corpus_chunk_size,
                **kwargs,
            )



        retriever = EvaluateRetrieval(model, score_function=score_function)  # or "cos_sim" or "dot"
        start_time = time()
        results = retriever.retrieve(corpus, queries)
        end_time = time()
        logger.info("Time taken to retrieve: {:.2f} seconds".format(end_time - start_time))

        ndcg, _map, recall, precision = retriever.evaluate(relevant_docs, results, retriever.k_values, ignore_identical_ids=kwargs.get("ignore_identical_ids", True))
        mrr = retriever.evaluate_custom(relevant_docs, results, retriever.k_values, "mrr")

        scores = {
            **{f"ndcg_at_{k.split('@')[1]}": v for (k, v) in ndcg.items()},
            **{f"map_at_{k.split('@')[1]}": v for (k, v) in _map.items()},
            **{f"recall_at_{k.split('@')[1]}": v for (k, v) in recall.items()},
            **{f"precision_at_{k.split('@')[1]}": v for (k, v) in precision.items()},
            **{f"mrr_at_{k.split('@')[1]}": v for (k, v) in mrr.items()},
        }

        return scores


class DRESModel:
    """
    Dense Retrieval Exact Search (DRES) in BeIR requires an encode_queries & encode_corpus method.
    This class converts a MTEB model (with just an .encode method) into BeIR DRES format.
    """

    def __init__(self, model, sep=" ", **kwargs):
        self.model = model
        self.sep = sep
        self.use_sbert_model = isinstance(model, SentenceTransformer)

    def encode_queries(self, queries: List[str], batch_size: int, **kwargs):
        if self.use_sbert_model:
            if isinstance(self.model._first_module(), Transformer):
                logger.info(f"Queries will be truncated to {self.model.get_max_seq_length()} tokens.")
            elif isinstance(self.model._first_module(), WordEmbeddings):
                logger.warning(
                    "Queries will not be truncated. This could lead to memory issues. In that case please lower the batch_size."
                )
        return self.model.encode(queries, batch_size=batch_size, **kwargs)

    def encode_corpus(self, corpus: List[Dict[str, str]], batch_size: int, **kwargs):
        if type(corpus) is dict:
            sentences = [
                (corpus["title"][i] + self.sep + corpus["text"][i]).strip()
                if "title" in corpus
                else corpus["text"][i].strip()
                for i in range(len(corpus["text"]))
            ]
        else:
            sentences = [
                (doc["title"] + self.sep + doc["text"]).strip() if "title" in doc else doc["text"].strip()
                for doc in corpus
            ]
        return self.model.encode(sentences, batch_size=batch_size, **kwargs)



from ...abstasks import AbsTaskClassification, MultilingualTask

_LANGUAGES = ["en", "de", "es", "fr", "ja", "zh"]


class AmazonReviewsClassification(MultilingualTask, AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "AmazonReviewsClassification",
            "hf_hub_name": "mteb/amazon_reviews_multi",
            "description": (
                "A collection of Amazon reviews specifically designed to aid research in multilingual text"
                " classification."
            ),
            "reference": "https://arxiv.org/abs/2010.02573",
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["validation", "test"],
            "eval_langs": _LANGUAGES,
            "main_score": "accuracy",
            "revision": "1399c76144fd37290681b995c656ef9b2e06e26d",
        }



import datasets

from mteb.abstasks import AbsTaskClassification


class NordicLangClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "NordicLangClassification",
            "hf_hub_name": "strombergnlp/nordic_langid",
            "description": "A dataset for Nordic language identification.",
            "reference": "https://aclanthology.org/2021.vardial-1.8/",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["da", "sv", "nb", "nn", "is", "fo"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 32,
            "revision": "e254179d18ab0165fdb6dbef91178266222bee2a",
        }

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"], "10k", revision=self.description.get("revision")  # select relevant subset
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self):
        self.dataset = self.dataset.rename_column("sentence", "text")
        self.dataset = self.dataset.rename_column("language", "label")



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class SickrPLSTS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "SICK-R-PL",
            "hf_hub_name": "PL-MTEB/sickr-pl-sts",
            "description": "Polish version of SICK dataset for textual relatedness.",
            "reference": "https://aclanthology.org/2020.lrec-1.207.pdf",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "cosine_spearman",
            "min_score": 1,
            "max_score": 5
        }


class CdscrSTS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "CDSC-R",
            "hf_hub_name": "PL-MTEB/cdscr-sts",
            "description": "Compositional Distributional Semantics Corpus for textual relatedness.",
            "reference": "https://aclanthology.org/P17-1073.pdf",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["pl"],
            "main_score": "cosine_spearman",
            "min_score": 1,
            "max_score": 5
        }
        


import random
from abc import ABC, abstractmethod

import datasets
import numpy as np
import torch


class AbsTask(ABC):
    def __init__(self, seed=42, **kwargs):
        self.dataset = None
        self.data_loaded = False
        self.is_multilingual = False
        self.is_crosslingual = False
        self.save_suffix = kwargs.get("save_suffix", "")

        self.seed = seed
        random.seed(self.seed)
        np.random.seed(self.seed)
        torch.manual_seed(self.seed)
        torch.cuda.manual_seed_all(self.seed)

    def load_data(self, **kwargs):
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        # TODO: add split argument
        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"], revision=self.description.get("revision", None)
        )
        self.data_loaded = True

    @property
    @abstractmethod
    def description(self):
        """
        Returns a description of the task. Should contain the following fields:
        name: Name of the task (usually equal to the class name. Should be a valid name for a path on disc)
        description: Longer description & references for the task
        type: Of the set: [sts]
        eval_splits: Splits used for evaluation as list, e.g. ['dev', 'test']
        main_score: Main score value for task
        """
        raise NotImplementedError

    @abstractmethod
    def evaluate(self, model, split="test"):
        """
        Evaluates a Sentence Embedding Model on the task.
        Returns a dict (that can be serialized to json).
        :param model: Sentence embedding method. Implements a encode(sentences) method, that encodes sentences
        and returns a numpy matrix with the sentence embeddings
        :param split: Which datasplit to be used.
        """
        raise NotImplementedError



from ...abstasks.AbsTaskClustering import AbsTaskClustering


class TwentyNewsgroupsClustering(AbsTaskClustering):
    @property
    def description(self):
        return {
            "name": "TwentyNewsgroupsClustering",
            "hf_hub_name": "mteb/twentynewsgroups-clustering",
            "description": "Clustering of the 20 Newsgroups dataset (subject only).",
            "reference": "https://scikit-learn.org/0.19/datasets/twenty_newsgroups.html",
            "type": "Clustering",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "v_measure",
            "revision": "6125ec4e24fa026cec8a478383ee943acfbd5449",
        }



from ...abstasks import AbsTaskClassification


class ToxicConversationsClassification(AbsTaskClassification):
    @property
    def description(self):
        return {
            "name": "ToxicConversationsClassification",
            "hf_hub_name": "mteb/toxic_conversations_50k",
            "description": (
                "Collection of comments from the Civil Comments platform together with annotations if the comment is"
                " toxic or not."
            ),
            "reference": (
                "https://www.kaggle.com/competitions/jigsaw-unintended-bias-in-toxicity-classification/overview"
            ),
            "category": "s2s",
            "type": "Classification",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 32,
            "revision": "d7c0de2777da35d6aae2200a62c6e0e5af397c4c",
        }



from ...abstasks.AbsTaskSTS import AbsTaskSTS


class STS14STS(AbsTaskSTS):
    @property
    def description(self):
        return {
            "name": "STS14",
            "hf_hub_name": "mteb/sts14-sts",
            "description": "SemEval STS 2014 dataset. Currently only the English dataset",
            "reference": "http://alt.qcri.org/semeval2014/task10/",
            "type": "STS",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 5,
            "revision": "6031580fec1f6af667f0bd2da0a551cf4f0b2375",
        }



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class DBPedia(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "DBPedia",
            "beir_name": "dbpedia-entity",
            "description": (
                "DBpedia-Entity is a standard test collection for entity search over the DBpedia knowledge base"
            ),
            "reference": "https://github.com/iai-group/DBpedia-Entity/",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }



import os
from .AbsTask import AbsTask


class BeIRPLTask(AbsTask):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def load_data(self, eval_splits=None, **kwargs):
        """
        Load dataset from BeIR-PL benchmark.
        """
        try:
            from beir.datasets.data_loader_hf import HFDataLoader
        except ImportError:
            raise Exception("Retrieval tasks require beir package. Please install it with `pip install mteb[beir]`")


        if self.data_loaded:
            return
        if eval_splits is None:
            eval_splits = self.description["eval_splits"]
        dataset = self.description["beir_name"]

        # cqadupstack not on huggingface yet
        # dataset, sub_dataset = dataset.split("/") if "cqadupstack" in dataset else (dataset, None)

        self.corpus, self.queries, self.relevant_docs = {}, {}, {}

        for split in eval_splits:

            corpus, queries, qrels = HFDataLoader(hf_repo=f"clarin-knext/{dataset}", streaming=False, keep_in_memory=False).load(split=split)
            # Conversion from DataSet
            queries = {query['id']: {'text': query['text']} for query in queries}
            corpus = {doc['id']: {'title': doc['title'] , 'text': doc['text']} for doc in corpus}

            self.corpus[split], self.queries[split], self.relevant_docs[split] = corpus, queries, qrels

        self.data_loaded = True



from ...abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from ...abstasks.BeIRTask import BeIRTask


class TRECCOVID(AbsTaskRetrieval, BeIRTask):
    @property
    def description(self):
        return {
            "name": "TRECCOVID",
            "beir_name": "trec-covid",
            "description": "TRECCOVID is an ad-hoc search challenge based on the CORD-19 dataset containing scientific articles related to the COVID-19 pandemic",
            "reference": "https://ir.nist.gov/covidSubmit/index.html",
            "description": "TRECCOVID is an ad-hoc search challenge based on the CORD-19 dataset containing scientific articles related to the COVID-19 pandemic.",
            "type": "Retrieval",
            "category": "s2p",
            "eval_splits": ["test"],
            "eval_langs": ["en"],
            "main_score": "ndcg_at_10",
        }
